// floortemp_tft_original_demo
//
// original sketch for the 10-sensor floortemp_tft sketch - featuring two circular segmented scales
// sketch displays data acquired by ten Dallas DS18B20 temperature sensors
// via a one-wire bus connected to pin 13 of an Arduino Nano 
// pull up resistor for the DS18B20 bus is fine tuned with a 10k pot meter (approx 4.7 kOhm is required)
//  
// sketch also  averages data from two DHT11 relative humidity sensors - DHT11 temperature sensing is ignored
// on Nano pins 12 and 19 (note: SD control pins on the TFT shield are borrowed for this purpose - SD not functional) 
// pull up resistor for DHT11's 10K ohm 
// pins 10 and 11 are available for additional sensoring e.g. a barometer bmp180
//
// 3.5 inch TFT screen ILI 9481 320 x 480 pixels portrait
//
// =============================================================================================================================
// THIS IS A FUNCTIONAL DEMO - there is in 'loop' a temporary parm block with fixed numeric values for the temp-humidity sensors
// =============================================================================================================================
//
// TFT control via pins A0-A4 and pins D2 thru D9 
// uses David Prentice's mcufriend_kbv library
//
// circular segmented scale and rainbow subroutines not mine - credit goes to to Bodmer and Danny van den Brande
// sketch by Floris Wouterlood
// public domain
// November 21, 2016
// sketch for Arduino Uno / Arduino Nano 
// display on a 3.5 320x480 color TFT screen with 0x9481 controller 
//
// by Floris Wouterlood November 21, 2016
// this is for the public domain
//
// here we go!

  #define LCD_CS A3              // Chip Select goes to Analog 3
  #define LCD_CD A2              // Command/Data goes to Analog 2
  #define LCD_WR A1              // LCD Write goes to Analog 1
  #define LCD_RD A0              // LCD Read goes to Analog 0
  #define LCD_RESET A4           // Can alternately just connect to Arduino's reset pin

  #include <DHT.h>               // DHT11 library
  #include <OneWire.h>           // one-wire library supporting the Dallas probe signal bus
  #include <DallasTemperature.h> // Dallas DS18B20 hardware-specific library
  #include <SPI.h>               // spi interface library for TFT
  #include <Adafruit_GFX.h>      // tft display hardware-specific library
  #include <MCUFRIEND_kbv.h>     // tft display hardware-specific library

// ========== assign human-readable names to some common 16-bit color values =========================
  #define	BLACK   0x0000
  #define	BLUE    0x001F
  #define	RED     0xF800
  #define	GREEN   0x07E0
  #define CYAN    0x07FF
  #define MAGENTA 0xF81F
  #define YELLOW  0xFFE0
  #define WHITE   0xFFFF

  int cursorx=0;                 // cursor positioning x coordinate
  int cursory=0;                 // cursor positioning y coordinate

    
// ========== Dallas DS18B20 ========================================================================= 
// single wire bus on pin 13 on the Arduino 
   #define ONE_WIRE_BUS 13
   
// ========== setup the TFT ==========================================================================
   MCUFRIEND_kbv tft;

// ========== setup a oneWire bus ====================================================================
   OneWire oneWire(ONE_WIRE_BUS);

// ========== pass oneWire reference to Dallas Temperature ===========================================
   DallasTemperature sensors(&oneWire);

// ========== probe addresses ========================================================================
// of the Dallas DS18B20
   
DeviceAddress probe_01 = { 0x28, 0xFF, 0x78, 0x1F, 0x92, 0x15, 0x03, 0x2F }; //ok
DeviceAddress probe_02 = { 0x28, 0xFF, 0xB0, 0x39, 0x92, 0x15, 0x03, 0x4A }; //ok

DeviceAddress probe_03 = { 0x28, 0xFF, 0x44, 0x32, 0x92, 0x15, 0x03, 0x67 }; //ok
DeviceAddress probe_04 = { 0x28, 0xFF, 0xE4, 0x27, 0x90, 0x15, 0x01, 0x1E }; //ok

DeviceAddress probe_05 = { 0x28, 0xFF, 0x11, 0x38, 0x92, 0x15, 0x03, 0x3B }; //ok
DeviceAddress probe_06 = { 0x28, 0xFF, 0x59, 0x2F, 0x92, 0x15, 0x04, 0xD1 }; //ok

DeviceAddress probe_07 = { 0x28, 0xFF, 0x63, 0x2E, 0x92, 0x15, 0x04, 0x67 }; //ok
DeviceAddress probe_08 = { 0x28, 0xFF, 0x33, 0x25, 0x90, 0x15, 0x01, 0x16 }; //ok

DeviceAddress probe_09 = { 0x28, 0xFF, 0x1B, 0x2E, 0x92, 0x15, 0x04, 0x34 }; //ok
DeviceAddress probe_10 = { 0x28, 0xFF, 0x37, 0x30, 0x92, 0x15, 0x03, 0x43 }; //ok

// ========== number of Dallas temperature sensors found =============================================
   int numberOfDevices;
   
// ========== DHT11s =================================================================================
// pins 12 and 19 (A5) available and wired with connector for sensors
// DHT11 relative humidity sensors  - pair mounted in pod - we average readings of this rather inaccurate senso   
   DHT sens1(12,DHT11);   //sensor one on pin 12 
   DHT sens2(19,DHT11);   //sensor two on pin 19 (A5)   
   

// variables used for control logic ==================================================================
   float Temp1, Temp2, Temp3, Temp4, Temp5;         //  storage DS18B20 variables temperature sensors
   float Temp6, Temp7, Temp8, Temp9, Temp10;        //  storage DS18B20 variables temperature sensors
   float DHT11_h1, DHT11_h2;                        //  storage variables DHT11 relative humidity values
   float humidity_av=0;                             //  storage variable for averaged relative humidity

void setup(void) { 
  
   uint16_t g_identifier;
   Serial.begin(9600);
   Serial.println("+------------------------------------+");
   Serial.println("+----- floortemp_tft_10_EN.ino ------+");
   Serial.println("+------------------------------------+");  
   Serial.println("English version  -  Public Domain");  
   Serial.println("Floris Wouterlood - November 21, 2016");
   Serial.println("");
   
   // Start the Dallas library
   Serial.println("starting one wire sensors");
   Serial.println("initializing Dallas Control Library");
   Serial.print("version ");
   Serial.println(DALLASTEMPLIBVERSION);
   sensors.begin();

   // locate DS18B20 devices on the oneWire bus
   Serial.println("");
   Serial.print(".. found ");
   Serial.print(sensors.getDeviceCount(), DEC);
   Serial.println(" DS18B20 devices ..........");
 
   // set the resolution to 10 bit (good enough?)
   sensors.setResolution(probe_01, 10);
   sensors.setResolution(probe_02, 10);
   sensors.setResolution(probe_03, 10);
   sensors.setResolution(probe_04, 10); 
   sensors.setResolution(probe_05, 10); 
   sensors.setResolution(probe_06, 10);
   sensors.setResolution(probe_07, 10);
   sensors.setResolution(probe_08, 10);
   sensors.setResolution(probe_09, 10);
   sensors.setResolution(probe_10, 10);
   
// provide identification TFT controller to library  
   g_identifier = 0x9481;  // this if for the 3.5 inch display ILI9341 controller
   tft.begin(g_identifier);

// ========== initialization message TFT screen ======================================================
   tft.fillScreen(BLUE);
   tft.setCursor(10, 10);
   tft.setTextColor(WHITE);  tft.setTextSize(1);
   tft.println("initializing. . . . . . .");
   delay(500);
   tft.fillScreen(BLUE);
   
// position blocks and texts in the TFT screen
// =========== temperature block section outlining ===================================================

   // draw upper rectangular line block for Title
   tft.drawLine(10,5,310,5,0xFFFF);     // color white hor
   tft.drawLine(10,5,10,40,0xFFFF);     // color white vert
   tft.drawLine(310,5,310,40,0xFFFF);   // color white vert
   tft.drawLine(10,40,310,40,0xFFFF);   // color white hor 

   // populate block with text
   tft.setTextColor(YELLOW);    
   tft.setTextSize(3);
   tft.setCursor(5, 10);
   tft.println(" floor heating");
 
   // underscore the title block with two magenta lines
   tft.drawLine(10,45,310,45,0xF81F);  // color magenta
   tft.drawLine(10,47,310,47,0xF81F);  // color magenta

   // second block: 'supply' and 'return'
   tft.setTextSize(3);
   tft.setTextColor(GREEN);  
   tft.setCursor(10, 70);   
   tft.println(" supply: ");
   cursorx=280;
   cursory=73;
   centigrade();

   tft.setTextSize(3);
   tft.setCursor(10, 95);   
   tft.println(" return: ");
   cursorx=280; cursory=98; 
   centigrade();   

   // line under aanvoer-retour block
   tft.drawLine(25,125,300,125,0xF81F);   // color magenta

   // populate the 'loop' blocks
   tft.setCursor(10, 140);     
   tft.println(" loop 1 in: ");
   cursorx=280; cursory=135; 
   centigrade();  
     
   tft.setCursor(10, 160);     
   tft.println(" loop 1 return");
   cursorx=280; cursory=155; 
   centigrade();  

   tft.setCursor(10, 190);     
   tft.println(" loop 2 in: ");
   cursorx=280; cursory=185; 
   centigrade();  
     
   tft.setCursor(10, 210);     
   tft.println(" loop 2 return: ");
   cursorx=280; cursory=205;     
   centigrade();  
   
   tft.setCursor(10, 240);   
   tft.println(" loop 3 in: ");
   cursorx=280; cursory=235; 
   centigrade(); 

   tft.setCursor(10, 260);     
   tft.println(" loop 3 return:");
   cursorx=280; cursory=255; 
   centigrade();  

   tft.setCursor(10, 290);     
   tft.println(" loop 4 in: ");
   cursorx=280; cursory=285; 
   centigrade(); 

   tft.setCursor(10, 310);     
   tft.println(" loop 4 return: ");
   cursorx=280; cursory=305; 
   centigrade();  

   
// ========== humidity block section outlining =======================================================

   // draw line block 
   tft.drawLine(5,355,315,355,0xFFFF);     // color white upper hor 
   tft.drawLine(5,388,315,388,0xFFFF);     // color white lower hor 
   tft.drawLine(5,355,5,388,0xFFFF);       // color white left vert
   tft.drawLine(315,355, 315,388,0xFFFF);   // color white right vert 

   tft.setTextColor(YELLOW);    
   tft.setTextSize(3);
   tft.setCursor(5,360);
   tft.println(" rel humidity");

    // line under block
   tft.drawLine(5,393,315,393,0xF81F);   // color magenta
   tft.drawLine(5,396,315,396,0xF81F);   // color magenta

   tft.setTextSize(2);
   tft.setTextColor(GREEN);  

   tft.setCursor(10, 410);   
   tft.println(" living room: ");
   tft.setCursor(280, 410);  
   tft.println("%");   


// ========== design and version block outlining and content =========================================
 
   tft.setTextSize(1);
   tft.setTextColor(YELLOW); 
   
   // draw line block 
   tft.drawLine(20,455,300,455,0xF81F);   // color magenta hor
   tft.drawLine(20,455,20,473,0xF81F);    // color magenta vert   
   tft.drawLine(300,455,300,473,0xF81F);  // color magenta vert     
   tft.drawLine(290,455,290,473,0xF81F);  // color magenta vert  
   tft.drawLine(20,473,300,473,0xF81F);   // color magenta hor 

   tft.setCursor(20,460);   
   tft.println(" FW / Sep 19, 2016 ");  
   delay(4000);

}


void loop (){

// ========== read DHT11s   ==========================================================================
// original instruction lines
   // 32 bytes is maximum payload
   char outBuffer[32]= "";
   char floatBuf1[10];
   char floatBuf2[10];
   float h, t;
 
// ========== read DHT11s ===========================================================================      

   // 32 bytes is maximum payload
   // read DHT11 sensors from buffer
      DHT11_h1= sens1.readHumidity();
      DHT11_h2= sens2.readHumidity();
   
    // calculate averages
      humidity_av = ((DHT11_h1+DHT11_h2)/2);    
     
// ========== read DS18B20s===========================================================================
   sensors.requestTemperatures();
 
// load variables with sensor values 
   Temp1 = sensors.getTempC(probe_01);
   Temp2 = sensors.getTempC(probe_02);
   Temp3 = sensors.getTempC(probe_03);
   Temp4 = sensors.getTempC(probe_04); 
   Temp5 = sensors.getTempC(probe_05);
   Temp6 = sensors.getTempC(probe_06);  
   Temp7 = sensors.getTempC(probe_07);
   Temp8 = sensors.getTempC(probe_08);  
   Temp9 = sensors.getTempC(probe_09);
   Temp10 = sensors.getTempC(probe_10);       

//========== first display DS18B20 readings on serial monitor ========================================


// ===================================== TEMPORARY PARM BLOCK FOR DEMO ===================================
// to provide nice TFT display - else -127 degrees for all DS18B20 sensors 

   DHT11_h1= 45.33;  // humidity 1 probe
   DHT11_h2= 58.66;  // humidity 2 probe
   humidity_av = ((DHT11_h1+DHT11_h2)/2); // necessary to calculate average humidity  

   Temp1 = 45;     // ds18b20_probe_01 on manifold supply CH 
   Temp2 = 21;     // ds18b20_probe_02 on return supply CH

   Temp3 = 23;     // ds18b20_probe_03 on loop 1 IN 
   Temp4 = 22;     // ds18b20_probe_04 on loop 2 return
   
   Temp5 = 23;     // ds18b20_probe_05 on loop 2 IN 
   Temp6 = 22;     // ds18b20_probe_06 on loop 2 return

   Temp7 = 23;     // ds18b20_probe_07 on loop 3 IN 
   Temp8 = 21;     // ds18b20_probe_08 on loop 3 return
   
   Temp9 = 23;     // ds18b20_probe_09 on loop 4 IN 
   Temp10 = 21;     // ds18b20_probe_10 on loop 4 return

// ===================================== END OF TEMPORARY PARM BLOCK  =====================================



// probe 01 = temperature CH suppy water manifold
      Serial.println(" ");
      Serial.println("======================================");  
      Serial.print("supply CH at manifold: ....  ");
      Serial.print(Temp1,1);
      Serial.println(" *C "); 
      cursorx=210; cursory=70;
      sticker();
      tft.setCursor(215,75);     
      tft.setTextColor(YELLOW); 
      tft.setTextSize(2);          
      tft.println(Temp1,1); // temp supply CH to manifold
      blip();  
      delay(1000);
      
// probe 02 = temperature return water from manifold back to furnace       
      Serial.print("return CH at manifold: ....  ");
      Serial.print(Temp2,1);
      Serial.println(" *C "); 
      Serial.println(" "); 
      cursorx=210; cursory=95;
      sticker();   
      tft.setCursor(215,95);
      tft.setTextColor(YELLOW); 
      tft.setTextSize(2);          
      tft.println(Temp2,1);
      blip();
      delay(1000); 

// probe 03 = temperature loop 1 in
      Serial.print("loop 1 in:  ...............  ");
      Serial.print(Temp3,1);
      Serial.println(" *C "); 
      cursorx=210; cursory=135;
      sticker();
      tft.setCursor(215,135);
      tft.setTextColor(YELLOW); 
      tft.setTextSize(2);          
      tft.println(Temp3,1);
      blip();  
      delay(1000); 
   
// probe 04 = temperature loop 1 out      
      Serial.print("loop 1 out: ...............  ");
      Serial.print(Temp4,1);
      Serial.println(" *C "); 
      Serial.println(" "); 
      cursorx=210; cursory=155;
      sticker(); 
      tft.setCursor(215,155);
      tft.setTextColor(YELLOW); 
      tft.setTextSize(2);          
      tft.println(Temp4,1);
      blip(); 
      delay(1000);

// probe 05 = temperature loop 2 in 
      Serial.print("loop 2 in:  ...............  "); 
      Serial.print(Temp5,1);
      Serial.println(" *C"); 
      cursorx=210; cursory=185;
      sticker(); 
      tft.setCursor(215,185);
      tft.setTextColor(YELLOW); 
      tft.setTextSize(2);          
      tft.println(Temp5,1);
      blip(); 
      delay(1000);  

// probe 06 = temperature loop 2 out
      Serial.print("loop 2 out: ...............  ");
      Serial.print(Temp6,1);
      Serial.println(" *C "); 
      Serial.println(" "); 
      cursorx=210; cursory=205;
      sticker();  
      tft.setCursor(215,205);
      tft.setTextColor(YELLOW); 
      tft.setTextSize(2);          
      tft.println(Temp6,1);
      blip();
      delay(1000);
  
// probe 07 = temperature loop 3 in       
      Serial.print("loop 3 in:  ...............  ");
      Serial.print(Temp7,1);
      Serial.println(" *C "); 
      cursorx=210; cursory=235;
      sticker(); 
      tft.setCursor(215,235);
      tft.setTextColor(YELLOW); 
      tft.setTextSize(2);          
      tft.println(Temp7,1);
      blip();  
      delay(1000);

// probe 08 = temperature loop 3 uout     
      Serial.print("loop 3 out: ...............  ");
      Serial.print(Temp8,1);
      Serial.println(" *C "); 
      Serial.println(" "); 
      cursorx=210; cursory=255;
      sticker(); 
      tft.setCursor(215,255);
      tft.setTextColor(YELLOW); 
      tft.setTextSize(2);          
      tft.println(Temp8,1);
      blip();  
      delay(1000); 

// probe 09 = temperature loop 4 in    
      Serial.print("loop 4 in:  ...............  ");
      Serial.print(Temp9,1);
      Serial.println(" *C "); 
      cursorx=210; cursory=285;
      sticker(); 
      tft.setCursor(215,285);
      tft.setTextColor(YELLOW); 
      tft.setTextSize(2);          
      tft.println(Temp9,1);
      blip();  
      delay(1000); 

// probe 10 = temperature loop 4 uit  
      Serial.print("loop 4 out: ...............  ");
      Serial.print(Temp10,1);
      Serial.println(" *C "); 
      Serial.println(" "); 
      cursorx=210; cursory=305;
      sticker(); 
      tft.setCursor(215,305);
      tft.setTextColor(YELLOW); 
      tft.setTextSize(2);          
      tft.println(Temp10,1);
      blip();  
      delay(1000); 


 // relative humidity
 // serial monitor part  

      Serial.println("======================================");  
      Serial.println("relative humidity sensors:");  
      Serial.print("sensor 1: ..................... ");
      Serial.print(DHT11_h1,0);
      Serial.println (" %");    
      Serial.print("sensor 2: ..................... ");
      Serial.print(DHT11_h2,0);
      Serial.println (" %");    
      Serial.print("average:   .................... ");
      Serial.print(humidity_av,0);
      Serial.println (" %");  
      
      Serial.println("======================================"); 
      Serial.println (" ");    
      Serial.println ("... next iteration ...");
      
// TFT part     
      cursorx=210; cursory=405;
      sticker();
      tft.setCursor(240,410);
      tft.setTextColor(YELLOW); 
      tft.setTextSize(2);   
      tft.println(humidity_av,0); 
      blip();  
      delay(1000);    

}

// ========== subroutines ============================================================================


void centigrade (){  // centigrade subroutine 
   tft.setCursor(cursorx,cursory); 
   tft.setTextSize(1);
   tft.println("o");
   tft.setCursor((cursorx+10),cursory); 
   tft.setTextSize(2);
   tft.println("C");
}


void sticker (){  // erases readings displayed previous cycle previous
  
       tft.fillRect(cursorx,cursory,55,20,0x001F); // blue   
   //  tft.fillRect(cursorx,cursory,55,20,0xF800); // red for testing  
   //  tft.fillRect(cursorx,cursory,55,20,0xF81F); // magenta for testing   
}


void blip (){ // shows a blinking 'activity' blip in the footer area, right 
 
    cursorx=293;
    cursory=460;
    tft.setTextSize(1);

    tft.setTextColor(YELLOW); 
    tft.setCursor(cursorx,cursory); 
    tft.println(char(177));
    delay(500); 
    tft.setTextColor(BLUE); 
    tft.setCursor(cursorx,cursory); 
    tft.println(char(177));
     
    tft.setTextSize(2);
  
}
