// floortemp_tft_updated_demo
//
// updated sketch for the 10-sensor floortemp_tft sketch - featuring two circular segmented scales
// sketch displays data acquired by ten Dallas DS18B20 temperature sensors
// via a one-wire bus connected to pin 13 of an Arduino Nano 
// pull up resistor for the DS18B20 bus is fine tuned with a 10k pot meter (approx 4.7 kOhm is required)
//  
// sketch also  averages data from two DHT11 relative humidity sensors - DHT11 temperature sensing is ignored
// on Nano pins 12 and 19 (note: SD control pins on the TFT shield are borrowed for this purpose - SD not functional) 
// pull up resistor for DHT11's 10K ohm 
// pins 10 and 11 are available for additional sensoring e.g. a barometer bmp180
//
// 3.5 inch TFT screen ILI 9481 320 x 480 pixels portrait
//
// =============================================================================================================================
// THIS IS A FUNCTIONAL DEMO - there is in 'loop' a temporary parm block with fixed numeric values for the temp-humidity sensors
// =============================================================================================================================
//
// TFT control via pins A0-A4 and pins D2 thru D9 
// uses David Prentice's mcufriend_kbv library
//
// circular segmented scale and rainbow subroutines not mine - credit goes to to Bodmer and Danny van den Brande
// sketch by Floris Wouterlood
// public domain
// Dec 12, 2019

   #include <OneWire.h>                                                                 // library
   #include <DHT.h>                                                                     // library
   #include <Adafruit_GFX.h>                                                            // library                             
   #include <MCUFRIEND_kbv.h>                                                           // library
   #include <DallasTemperature.h>                                                       // library
   
// Dallas DS18B20 section
   #define ONE_WIRE_BUS_PIN 13
   OneWire oneWire (ONE_WIRE_BUS_PIN);
   DallasTemperature sensors (&oneWire);

// these are the floor loops ds18b20s
   DeviceAddress ds18b20_probe_01 = { 0x28, 0xFF, 0x78, 0x1F, 0x92, 0x15, 0x03, 0x2F }; //ok supply pipe CH 
   DeviceAddress ds18b20_probe_02 = { 0x28, 0xFF, 0xB0, 0x39, 0x92, 0x15, 0x03, 0x4A }; //ok return pipe CH
 
   DeviceAddress ds18b20_probe_03 = { 0x28, 0xFF, 0x44, 0x32, 0x92, 0x15, 0x03, 0x67 }; //ok loop 1 IN 
   DeviceAddress ds18b20_probe_04 = { 0x28, 0xFF, 0xE4, 0x27, 0x90, 0x15, 0x01, 0x1E }; //ok loop 1 return

   DeviceAddress ds18b20_probe_05 = { 0x28, 0xFF, 0x11, 0x38, 0x92, 0x15, 0x03, 0x3B }; //ok loop 2 IN
   DeviceAddress ds18b20_probe_06 = { 0x28, 0xFF, 0x59, 0x2F, 0x92, 0x15, 0x04, 0xD1 }; //ok loop 2 return

   DeviceAddress ds18b20_probe_07 = { 0x28, 0xFF, 0x63, 0x2E, 0x92, 0x15, 0x04, 0x67 }; //ok loop 3 IN
   DeviceAddress ds18b20_probe_08 = { 0x28, 0xFF, 0x33, 0x25, 0x90, 0x15, 0x01, 0x16 }; //ok loop 3 return

   DeviceAddress ds18b20_probe_09 = { 0x28, 0xFF, 0x1B, 0x2E, 0x92, 0x15, 0x04, 0x34 }; //ok loop 4 IN
   DeviceAddress ds18b20_probe_10 = { 0x28, 0xFF, 0x37, 0x30, 0x92, 0x15, 0x03, 0x43 }; //ok loop 4 return
   
   MCUFRIEND_kbv tft;
  
   #define LCD_CS A3 
   #define LCD_CD A2 
   #define LCD_WR A1 
   #define LCD_RD A0 
   #define LCD_RESET A4 

// some principal color definitions
// RGB 565 color picker at https://ee-programming-notepad.blogspot.com/2016/10/16-bit-color-generator-picker.html
   #define WHITE       0xFFFF
   #define BLACK       0x0000
   #define BLUE        0x001F
   #define RED         0xF800
   #define GREEN       0x07E0
   #define CYAN        0x07FF
   #define MAGENTA     0xF81F
   #define YELLOW      0xFFE0
   #define GREY        0x2108 
   #define SCALE0      0x528A                                                  // inward pointing triangle scale segment  - color close to black 
   #define SCALE1      0x3151                                                  // outward pointing triangle scale segment - color deep blue 
   #define TEXT_COLOR  0xFFFF                                                  // is currently white 

// circular scale color scheme  
   #define RED2RED 0
   #define GREEN2GREEN 1
   #define BLUE2BLUE 2
   #define BLUE2RED 3
   #define GREEN2RED 4
   #define RED2GREEN 5

   uint32_t runTime = -99999;                                                  // time for next update
   int reading = 0;                                                            // value to be displayed in circular scale
   int reading1= 0;                                                            // value to be displayed in circular scale 
   boolean alert = 0;
   int8_t ramp = 1;
   int tesmod = 0;
   int xpos= 0;
   float temp_00, temp_01,temp_02, temp_03, temp_04, temp_05,temp_06, temp_07, temp_08, temp_09, temp_10;
   float h,t;                                                                  // DHT11s

// DHT11 relative humidity sensors  - pair mounted in pod - we average readings of this rather inaccurate sensor
   DHT sens1(12,DHT11);                                                        // sensor one on pin 12  
   DHT sens2(19,DHT11);                                                        // sensor two on pin 19   
   float DHT11_h1, DHT11_h2;                                                   //  storage variables DHT11 relative humidity values
   float humidity_av=0;                                                        //  storage variable for averaged relative humidity
 

void setup() {

// Dallas DS18B20 section
   sensors.begin ();
   sensors.setResolution (ds18b20_probe_01, 10);
   sensors.setResolution (ds18b20_probe_02, 10);
   sensors.setResolution (ds18b20_probe_03, 10);
   sensors.setResolution (ds18b20_probe_04, 10); 
   sensors.setResolution (ds18b20_probe_05, 10); 
   sensors.setResolution (ds18b20_probe_06, 10);
   sensors.setResolution (ds18b20_probe_07, 10);
   sensors.setResolution (ds18b20_probe_08, 10);
   sensors.setResolution (ds18b20_probe_09, 10);
   sensors.setResolution (ds18b20_probe_10, 10);

// TFT controller detection
   uint16_t ID;
   ID = tft.readID ();                                                          
   Serial.begin (9600);
   Serial.println ("starting TFT display");
   Serial.print  ("controller ID = 0x");
   Serial.println (ID,HEX); 
   Serial.println ("Starting 1-wire sensors");
   Serial.print ("Temperature Control Library, version ");
   Serial.println (DALLASTEMPLIBVERSION);
   sensors.begin ();
 
// locate devices on the one wire bus
   Serial.println ("");
   Serial.print ("found ");
   Serial.print (sensors.getDeviceCount(), DEC);
   Serial.println (" devices");

// static elements to be continuously displayed 
   tft.reset ();                                                               // reset TFT and begin
   tft.begin (ID);  
   tft.setRotation (0);                                                        // display in portrait
   tft.fillScreen (BLACK); 
   tft.drawRect (0,0,320,480,BLUE);                                            // draw outer frame as a line
   tft.drawLine (0,37,320,37,BLUE);                                            // draw blue line grid
   tft.drawLine (0,240,320,240,BLUE);
   tft.drawLine (0,275,320,275,BLUE);
   tft.drawLine (200,420,320,420,BLUE);                                        // horizontal upper border rel humidity block
   tft.drawLine (200,0,200,480,BLUE);                                          // vertical column separator 
 
   tft.setTextColor (TEXT_COLOR,BLACK);                                        // print labels on TFT
   tft.setTextSize (2);
   tft.setCursor (70,14);                                          
   tft.print ("SUPPLY");
   tft.setCursor (70,252);                                         
   tft.print ("RETURN");
   tft.setCursor (233,14);                                         
   tft.print ("Loops");
   tft.setCursor (233,252);   
   tft.print ("Loops");
   tft.setTextSize (1);
   tft.setCursor (228,36);                                         
   tft.print (" (floor-in) ");
   tft.setCursor (235,274);   
   tft.print (" (return) ");
   
   tft.setTextColor (TEXT_COLOR,BLACK);  
   tft.setTextSize (2); 
   tft.setCursor (230,430);                                                    // DHT11 label in yellow
   tft.print ("DHT11s");
   tft.setTextSize (2); 
   tft.setCursor (230,70);                                                     // loops 1-4 water flows into floor
   tft.print ("1:");
   tft.setCursor (230,100);   
   tft.print ("2:");
   tft.setCursor (230,130);   
   tft.print ("3:");
   tft.setCursor (230,160);   
   tft.print ("4:");
   tft.setCursor (230,300);                                                    // loops 1-4 water flows out of floor
   tft.print ("1:");
   tft.setCursor (230,330);   
   tft.print ("2:");
   tft.setCursor (230,360);   
   tft.print ("3:");
   tft.setCursor (230,390);   
   tft.print ("4:"); 
}

void loop() 
{

// read DHT11s
   char outBuffer[32]= "";                                                     // 32 bytes is maximum payload
   char floatBuf1[10];
   char floatBuf2[10];
   DHT11_h1= sens1.readHumidity();    
   DHT11_h2= sens2.readHumidity();     
   humidity_av = ((DHT11_h1+DHT11_h2)/2);  

   delay(1000);
  
   sensors.requestTemperatures ();                                             // instruct all devices on the bus to read temperature
   temp_01 = sensors.getTempC (ds18b20_probe_01);  
   temp_02 = sensors.getTempC (ds18b20_probe_02);
   temp_03 = sensors.getTempC (ds18b20_probe_03);  
   temp_04 = sensors.getTempC (ds18b20_probe_04);
   temp_05 = sensors.getTempC (ds18b20_probe_05);  
   temp_06 = sensors.getTempC (ds18b20_probe_06);
   temp_07 = sensors.getTempC (ds18b20_probe_07);  
   temp_08 = sensors.getTempC (ds18b20_probe_08);
   temp_09 = sensors.getTempC (ds18b20_probe_09);  
   temp_10 = sensors.getTempC (ds18b20_probe_10);



// ===================================== TEMPORARY PARM BLOCK FOR DEMO ===================================
// to provide nice TFT display - else -127 degrees for all DS18B20 sensors 

   DHT11_h1= 45.33;  // humidity 1 probe
   DHT11_h2= 58.66;  // humidity 2 probe
   humidity_av = ((DHT11_h1+DHT11_h2)/2); // necessary to calculate average humidity  

   temp_01 = 45;     // ds18b20_probe_01 on manifold supply CH 
   temp_02 = 21;     // ds18b20_probe_02 on return supply CH

   temp_03 = 23;     // ds18b20_probe_03 on loop 1 IN 
   temp_04 = 22;     // ds18b20_probe_04 on loop 2 return
   
   temp_05 = 23;     // ds18b20_probe_05 on loop 2 IN 
   temp_06 = 22;     // ds18b20_probe_06 on loop 2 return

   temp_07 = 23;     // ds18b20_probe_07 on loop 3 IN 
   temp_08 = 21;     // ds18b20_probe_08 on loop 3 return
   
   temp_09 = 23;     // ds18b20_probe_09 on loop 4 IN 
   temp_10 = 21;     // ds18b20_probe_10 on loop 4 return

// ===================================== END OF TEMPORARY PARM BLOCK  =====================================

   Serial.print ("CH-supply - probe 1:   ");
   Serial.println (temp_01,1);
   Serial.print ("CH-return - probe 2:   ");
   Serial.println (temp_02,1);
   Serial.println ();
   Serial.println ("lussen:   ");  
   Serial.print ("probe 3:   ");
   Serial.println (temp_03,1); 
   Serial.print ("probe 4:   ");
   Serial.println (temp_04,1);
   Serial.print ("probe 5:   ");
   Serial.println (temp_05,1);
   Serial.print ("probe 6:   ");
   Serial.println (temp_06,1); 
   Serial.print ("probe 7:   ");
   Serial.println (temp_07,1); 
   Serial.print ("probe 8:   ");
   Serial.println (temp_08,1);
   Serial.print ("probe 9:   ");
   Serial.println (temp_09,1);
   Serial.print ("probe 10:  ");
   Serial.println (temp_10,1); 
   Serial.println (); 
   Serial.println("relative humidity:");  
   Serial.print("sensor 1: ");
   Serial.print(DHT11_h1,0);
   Serial.println (" %");    
   Serial.print("sensor 2: ");
   Serial.print(DHT11_h2,0);
   Serial.println (" %");    
   Serial.print("average:  ");
   Serial.print(humidity_av,0);
   Serial.println (" %");  
   
   Serial.print ("end of cycle");  
   Serial.println ();
   Serial.println("==========================");  
   Serial.println ();
          
// print numerical loop temperatures to TFT display right column
   tft.setTextColor (YELLOW,BLACK);  
   tft.setTextSize (2); 
   tft.setCursor (260,70);                                                    
   tft.print (temp_03,1);                                                      // loop 1 IN - temp_03
   tft.setCursor (260,100);                                                    
   tft.print (temp_05,1);                                                      // loop 2 IN - temp_05
   tft.setCursor (260,130);                                                    
   tft.print (temp_07,1);                                                      // loop 3 IN - temp_07
   tft.setCursor (260,160);                                                    
   tft.print (temp_09,1);                                                      // loop 4 IN - temp_09
   tft.setCursor (260,300);                                                    
   tft.print (temp_04,1);                                                      // loop 1 RETURN - temp_04
   tft.setCursor (260,330);                                                    
   tft.print (temp_06,1);                                                      // loop 2 RETURN - temp_06
   tft.setCursor (260,360);                                                    
   tft.print (temp_08,1);                                                      // loop 3 RETURN - temp_08
   tft.setCursor (260,390);                                                    
   tft.print (temp_10,1);                                                      // loop 4 RETURN - temp_10 
   
   tft.setCursor (245, 455); 
   tft.print (humidity_av,0);                                                  // average of relative humidity
   tft.print (" %");            
   tft.setTextColor (TEXT_COLOR,BLACK);  
   
   
// fill the two ring scales with inlet and outlet temperatures           
   if (millis () - runTime >= 500)                                            
     {                                    
      runTime = millis ();
      if( tesmod==0)
        {
         reading =  99;
         reading1 = 99;
        }

// ====================== fill the upper ring = temp out ==========================================================================  
      if( tesmod==1)
        {
         reading = temp_01; // important: here ring is seeded with value      
        } 
   int xpos = 20, ypos = 50, gap = 100, radius = 85;                           // position of upper ring and proportion
   ringMeter (reading,0,100, xpos,ypos,radius,"Celsius",GREEN2RED);            // draw analog meter
   

// ====================== fill the lower ring = temp in  ==========================================================================   
      if (tesmod==1)
        {
         reading = temp_02; // important: here ring is seeded with value -- inlet  dus moet worden temp_01 want probe_01
        }  
   xpos = 20, ypos = 290, gap = 100, radius = 85;                              // position of lower ring and proportion
   ringMeter (reading,0,100, xpos,ypos,radius,"Celsius",GREEN2RED);            // draw analogue meter

   tesmod = 1;
  }
}

// ################################################################################################################################
//  Draw the meter on the screen, returns x coord of righthand side
// ################################################################################################################################

int ringMeter(int value, int vmin, int vmax, int x, int y, int r, char *units, byte scheme)
{
// Minimum value of r is about 52 before value text intrudes on ring
// drawing the text first is an option
 
   x += r; y += r;                                                             // calculate coordinates of center of ring
   int w = r / 3;                                                              // width of outer ring is 1/4 of radius
   int angle = 150;                                                            // half the sweep angle of the meter (300 degrees)
   int v = map(value, vmin, vmax, -angle, angle);                              // map the value to an angle v
   byte seg = 3;                                                               // segments are 3 degrees wide = 100 segments for 300 degrees
   byte inc = 6;                                                               // draw segments every 3 degrees, increase to 6 for segmented ring
   int colour = BLUE;                                                          // variable to save "value" text color from scheme and set default
 
 
   for (int i = -angle+inc/2; i < angle-inc/2; i += inc)                       // draw color blocks every increment degrees
     {           
      float sx = cos((i - 90) * 0.0174532925);                                 // calculate pair of coordinates for segment start
      float sy = sin((i - 90) * 0.0174532925);
      uint16_t x0 = sx * (r - w) + x;
      uint16_t y0 = sy * (r - w) + y;
      uint16_t x1 = sx * r + x;
      uint16_t y1 = sy * r + y;
    
      float sx2 = cos((i + seg - 90) * 0.0174532925);                          // salculate pair of coordinates for segment end
      float sy2 = sin((i + seg - 90) * 0.0174532925);
      int x2 = sx2 * (r - w) + x;
      int y2 = sy2 * (r - w) + y;
      int x3 = sx2 * r + x;
      int y3 = sy2 * r + y;

      if (i < v) 
         {                                                                     // fill in coloured segments with 2 triangles
          switch (scheme)
             {
              case 0: colour = RED; break;                                     // fixed color
              case 1: colour = GREEN; break;                                   // fixed color
              case 2: colour = BLUE; break;                                    // fixed colour
              case 3: colour = rainbow(map(i, -angle, angle, 0, 127)); break;  // full spectrum blue to red
              case 4: colour = rainbow(map(i, -angle, angle, 70, 127)); break; // green to red (high temperature etc)
              case 5: colour = rainbow(map(i, -angle, angle, 127, 63)); break; // red to green (low battery etc)
              default: colour = BLUE; break;                                   // fixed color
             }
              tft.fillTriangle(x0, y0, x1, y1, x2, y2, colour);
              tft.fillTriangle(x1, y1, x2, y2, x3, y3, colour);
             }
      else                                                                     // fill in blank segments
             {
              tft.fillTriangle(x0, y0, x1, y1, x2, y2, SCALE1);                // color of unoccupied ring scale segment triangle pointing outward
              tft.fillTriangle(x1, y1, x2, y2, x3, y3, SCALE0);                // color of unoccupied ring scale segment traiangle pointing inward
             }
             }

   char buf [10];                                                              // convert value to a string
   byte len = 2; if (value > 999) len = 4;
   dtostrf (value, len, 0, buf);
   buf[len] = ' '; buf[len] = 0;                                               // add blanking space and terminator, helps to centre text too!
  
   tft.setTextSize (2);
   if (value>9)
      {
       tft.setTextColor (colour,BLACK);
       tft.setCursor (x-25,y-10);tft.setTextSize (5);
       tft.print (buf);
      }
  
   if(value<10)
      {
       tft.setTextColor (colour,BLACK);
       tft.setCursor (x-25,y-10);tft.setTextSize (5);
       tft.print (buf);
       }
  
   tft.setTextColor (WHITE,BLACK);
   tft.setCursor (x-39,y+75);tft.setTextSize (2);                              // units position relative to scale
   tft.print (units);                                                          // units display = celsius                                        
   return x + r;                                                               // calculate and return right hand side x coordinate
}

// ################################################################################################################################
// Return a 16 bit rainbow colour
// ################################################################################################################################

unsigned int rainbow(byte value)                                               // value is expected to be in range 0-127
{                                                                              // value is converted to a spectrum color from 0 = blue through to 127 = red
   byte red = 0;                                                               // red is the top 5 bits of a 16 bit colour value
   byte green = 0;                                                             // green is the middle 6 bits
   byte blue = 0;                                                              // blue is the bottom 5 bits
   byte quadrant = value / 32;

   if (quadrant == 0)
     {
      blue = 31;
      green = 2 * (value % 32);
      red = 0;
     }
   if (quadrant == 1)
     {
      blue = 31 - (value % 32);
      green = 63;
      red = 0;
     }
   if (quadrant == 2)
     {
      blue = 0;
      green = 63;
      red = value % 32;
     }
   if (quadrant == 3)
     {
      blue = 0;
      green = 63 - 2 * (value % 32);
      red = 31;
     }
   return (red << 11) + (green << 5) + blue;
}

// ################################################################################################################################
// Return a value in range -1 to +1 for a given phase angle in degrees
// ################################################################################################################################

float sineWave(int phase) 
{
  return sin(phase * 0.0174532925);
}


void printTemperature(DeviceAddress deviceAddress)
{
  float tempC = sensors.getTempC(deviceAddress);
    Serial.print ("C: ");
    Serial.print (tempC);
  
  if (tempC == -127.00)
    {
     Serial.print("Error getting temperature");
    } 
    else
    {
     Serial.print("C: ");
     Serial.print(tempC);
    }
}
